#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    DesktopBasket is free software; you can redistribute it and/or modify #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    DesktopBasket is distributed in the hope that it will be useful,      #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# This file contains main classes for DesktopBasket

import karamba, os, sys, locale, time, re
try:
	from commands import getoutput
	from xml.dom.minidom import parse
	from HTMLParser import HTMLParser
	from gettext import GNUTranslations
	from StringIO import StringIO

# Monitor files
	import gamin
# PyKde and PyQt
	from kdecore import *
	from kdeui import KPassivePopup, KDialog
	from kio import KService, KFileItem
	from qt import QPixmap, QFont

	from desktopbasket_struc import *

except ImportError, e:
	comm = "kdialog --caption DesktopBasket --error \"Failed to load a module, check your installation : \n" + str(e) + "\""
	karamba.execute(comm)
	sys.exit()

	
# Translation function
_ = lambda(x): x
theConfig = None

class DesktopBasket:

	def __init__(self, widget):

		global theConfig
		
		# currently displayed post-it
		self.currentPostIt = 0
		# to force refresh when theme is being maximized and was minimized
		# for too long
		self.skippedOnce = False
		# counter for refreshing notes
		self.cmpt_refresh=2
		# note for which the highlight is being changed
		self.hlCurrentNote=None
		# True if the display must be refreshed
		self.needRefresh = True

		print "Starting init..."
	
		theConfig = Desktopbasket_config(widget)	# configuration of the theme

		init_language(widget)
		self.dispElmt = DisplayElements(widget)	# display elements like buttons, images, ...
		
		self.notesGatherer = NotesGatherer()		# reads baskets and notes content
		self.actions = Desktopbasket_actions()		# actions triggered by the karamba.executeInteractive command

		if not theConfig.desiredTag:	# ask for which tags to use to display notes
			ans = os.spawnlp(os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _("Do you want to create and use the \"Desktop\" tag\nto display Basket's notes ?"), '--caption', 'DesktopBasket')
			if ans :
				theConfig.desiredTag=[]
				self.askForDesiredTag(widget)
			else:
				self.createDesktopTag(widget, False)

		if not theConfig.quickNotesBasket:
			theConfig.quickNotesBasket=''

		self.myPostIts = []		# all post-it
		tmpSmall = theConfig.smallNote
		theConfig.smallNote = False
		theConfig.smallNote = tmpSmall
		if theConfig.smallNote:
			karamba.resizeWidget(widget, theConfig.size[0], 20)
		karamba.changeTextColor(widget, self.dispElmt.title, int((Desktopbasket_config.colors[theConfig.defaultColor])[0:3]), int((Desktopbasket_config.colors[theConfig.defaultColor])[3:6]), int((Desktopbasket_config.colors[theConfig.defaultColor])[6:9]))
		print "Init done"

	def reloadNotes(self, widget):

		# if the post-it is reduced, no need to update
		if theConfig.smallNote:
			self.skippedOnce = True
			return
		self.skippedOnce = False

		# gathers notes
		notesTmp = self.notesGatherer.main()

		# if nothing changed since last reading,no need to update
		if not self.notesGatherer.needUpdate:
			#print "Notes didn\'t change since last update"
			return

		print "Notes changed recently, update needed"
		# Delete all current notes
		for pi in self.myPostIts:
			pi.delete(widget)

		self.myPostIts = []
		myNotes = notesTmp
		vpos = 20
		p = PostIt(theConfig)
		self.myPostIts.append(p)
		prev_basket=None
		for n in myNotes:
			# if basket's name must be displayed just before the note
			if theConfig.basketNameColor!='Don\'t show' and n.basket != prev_basket:
				prev_basket=n.basket
				kbn = karambaBasketName(widget, n, vpos, theConfig)
				vpos = vpos + karamba.getRichTextSize(widget, kbn.rt)[1]
			else:
				kbn=None
			kn = karambaNote(widget, n, vpos, theConfig, self.dispElmt)
			vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3	# vertical position
			if vpos > theConfig.size[1]-theConfig.yborder: 	# end of note is out of the post-it
				if p.notes!=[]:	# post-it is not empty
					p = PostIt(theConfig)	# create a new Post-It
					vpos = 20
					if theConfig.basketNameColor!='Don\'t show':
						if kbn:
							kbn.move(widget, vpos)
						else:
							kbn = karambaBasketName(widget, n, vpos, theConfig)
						vpos = vpos + karamba.getRichTextSize(widget, kbn.rt)[1]+3
						p.append(kbn)
					kn.move(widget, vpos)
					vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3
					self.myPostIts.append(p)	# and add it to the post-it list
				p.append(kn)
				if vpos > theConfig.size[1]-theConfig.yborder:
					# the note is too large to be displayed entirely
					p.setExtendable(widget, self.dispElmt.iconLoader)
			else:
				if kbn:
					p.append(kbn)
				p.append(kn)
		if len(self.myPostIts)<=0:
			self.currentPostIt = 0
		elif len(self.myPostIts)<=self.currentPostIt:
			self.currentPostIt = len(self.myPostIts)-1
		#self.updateCurrentPostIt(widget)
		self.notesGatherer.needUpdate = False
		self.needRefresh = True

	def update(self, widget):
		pi_index = 0
		self.reloadNotes(widget)
		if self.needRefresh:
			for pi in self.myPostIts:
				pi.show(widget, pi_index==self.currentPostIt)
				pi_index = pi_index+1
			self.needRefresh = False
		self.updateTitle(widget)

	# Open help file in a navigator
	def displayHelp(self):
		command = 'kfmclient exec '+theConfig.helpPath()
		karamba.execute(command)

	def updateTitle(self, widget):
		if theConfig.titleType=='Date':
			date = time.localtime()
			theConfig.titleText = time.strftime('%X', date)+'  '+ time.strftime('%x', date)
			karamba.changeText(widget, self.dispElmt.title, theConfig.titleText+" ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')
		karamba.changeText(widget, self.dispElmt.title, theConfig.titleText+" ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')

	def onMenuClick(self, widget, menu, id):
		# Select quick notes basket
		if id==self.dispElmt.configOptions['quickBasket']:
			self.askForQuickNotesBasket(widget)
		# Create a new text note
		if id==self.dispElmt.basketOptions['createNote']:
			if not isBasketRunning():
				command = ["kdialog", '--caption', 'DesktopBasket',"--error", _('Basket is not running')]
				karamba.executeInteractive(widget, command)
			elif theConfig.quickNotesBasket=="":
				command = ["kdialog", '--caption', 'DesktopBasket',"--error", _('You have not chosen a basket \nwhere to store quick notes.')]
				karamba.executeInteractive(widget, command)
			else:
				#command = ["kdialog", '--caption', 'DesktopBasket',"--textinputbox", _('Enter note\'s text')]
				command = ['python', theConfig.editorPath()+'/NotesEditor.py']
				self.actions.create_note_pid = karamba.executeInteractive(widget, command)
		# Launch Basket
		elif id==self.dispElmt.basketOptions['runBasket']:
			runBasket()
		# change title
		elif id==self.dispElmt.configOptions['title']:
			inputtext = karamba.readConfigEntry(widget, 'title')
			command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new title'), inputtext]
			self.actions.select_title_pid = karamba.executeInteractive(widget, command)
		# select text default colour
		elif id==self.dispElmt.configOptions['color']:
			command = ["kdialog", '--caption', 'DesktopBasket',"--combobox", _('Select text default color')]
			cTrans=[]
			for c in Desktopbasket_config.colors.keys():
				cTrans.append( _(c) )
			cTrans.sort()
			command.extend(cTrans)
			self.actions.select_color_pid = karamba.executeInteractive(widget, command)
		# Show/hide tags
		elif id==self.dispElmt.configOptions['showTags']:
			theConfig.showTags = not theConfig.showTags
			self.notesGatherer.needUpdate=True
			karamba.writeConfigEntry(widget, "showTags", str(theConfig.showTags))
		# Show/Remove date in title
		elif id==self.dispElmt.configOptions['titleDate']:
			if theConfig.titleType=='Date':
				theConfig.titleType= 'Text'
				theConfig.titleText = karamba.readConfigEntry(widget, "title")
				if not theConfig.titleText:
					theConfig.titleText="DesktopBasket"
				karamba.changeText(widget, self.dispElmt.title, theConfig.titleText+" ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')
			else:
				theConfig.titleType= 'Date'
			karamba.writeConfigEntry(widget, "titleType", theConfig.titleType)
		# change size
		elif id==self.dispElmt.configOptions['size']:
			s = str(theConfig.size[0])+' x '+str(theConfig.size[1])
			command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new size'), s]
			self.actions.select_size_pid = karamba.executeInteractive(widget, command)
		#change background
		elif id==self.dispElmt.configOptions['background']:
			pth = theConfig.imagesPath()
			command = ["kdialog", '--caption', 'DesktopBasket',"--getopenfilename", pth]
			self.actions.select_img_pid = karamba.executeInteractive(widget, command)
		# change title bar background
		elif id==self.dispElmt.configOptions['titleBar']:
			pth = theConfig.imagesPath()
			command = ["kdialog", '--caption', 'DesktopBasket', "--getopenfilename", pth]
			self.actions.select_ttBar_pid = karamba.executeInteractive(widget, command)
		# choose selecting tag
		elif id==self.dispElmt.configOptions['selecttag']:
			self.askForDesiredTag(widget)
		# use default backgrounds
		elif id==self.dispElmt.configOptions['resetTheme']:
			theConfig.bgImgFile = theConfig.imagesPath()+'/background.png'
			karamba.deleteImage(widget, self.dispElmt.bgImg)
			self.dispElmt.bgImg = karamba.createBackgroundImage(widget, 0, 20, theConfig.bgImgFile)
			karamba.resizeImage(widget, self.dispElmt.bgImg, theConfig.size[0], theConfig.size[1]-20)
			karamba.writeConfigEntry(widget, "bgImg", theConfig.bgImgFile)
			theConfig.ttBarImgFile = theConfig.imagesPath()+'/TitleBar.png'
			karamba.deleteImage(widget, self.dispElmt.ttBarImg)
			self.dispElmt.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, theConfig.ttBarImgFile)
			karamba.resizeImage(widget, self.dispElmt.ttBarImg, theConfig.size[0], 20)
			karamba.writeConfigEntry(widget, "titleBar", theConfig.ttBarImgFile)
			karamba.redrawWidget(widget)
		# force refresh
		elif id==self.dispElmt.configOptions['refresh']:
			self.notesGatherer.needUpdate=True
			self.reloadNotes(widget)
			karamba.redrawWidget(widget)
		# choose refresh rate
		elif id==self.dispElmt.configOptions['refreshRate']:
			command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new refresh rate (s)'), str(theConfig.refreshRate)]
			self.actions.select_refresh_pid = karamba.executeInteractive(widget, command)
		# choose baskets names colour
		elif id==self.dispElmt.configOptions['basketNameColor']:
			command = ["kdialog", '--caption', 'DesktopBasket',"--combobox", _('Select basket name color')]
			cTrans=[_('Don\'t show')]
			for c in Desktopbasket_config.colors.keys():
				cTrans.append( _(c) )
			cTrans.sort()
			command.extend(cTrans)
			self.actions.select_basketnamecolor_pid = karamba.executeInteractive(widget, command)
		# show about dialog
		elif id==self.dispElmt.configOptions['about']:
			aboutString = '\"DesktopBasket '+Desktopbasket_config.version+'\n'+_('Created by')+' :\n'+\
			Desktopbasket_config.authors+'\n\n'+_('Contributors')+' :\n'+\
			Desktopbasket_config.contributors+'\"'
			command = 'kdialog --caption DesktopBasket --msgbox '+aboutString+' --title \"'+ _('About')+'\"'
			karamba.execute(command)
		# help
		elif id==self.dispElmt.configOptions['help']:
			self.displayHelp()
		#create desktop tag
		elif id==self.dispElmt.configOptions['createDesktopTag']:
			self.createDesktopTag(widget, True)
		# choose highlight colour
		else:
			for color in self.dispElmt.highlightOptions.keys():
				if id==self.dispElmt.highlightOptions[color]:
					self.hlCurrentNote.changeHighlight(widget, color)
					break

	def onMeterClick(self, widget, meter, button):
		
		if button==1:	#left button
			if meter == self.dispElmt.moveLeftButton or meter == self.dispElmt.moveRightButton:	# arrows to show other post-it
				if theConfig.smallNote:	# don't update if the desktopbasket is reduced
					return
				if meter == self.dispElmt.moveLeftButton:	# display previous post-it
					self.currentPostIt = (self.currentPostIt-1)%len(self.myPostIts)
				elif meter == self.dispElmt.moveRightButton :	# display next post-it
					self.currentPostIt = (self.currentPostIt+1)%len(self.myPostIts)
				#self.updateCurrentPostIt(widget)
				self.needRefresh = True
			elif meter == self.dispElmt.basketButton:	# basket button pressed
				karamba.popupMenu(widget, self.dispElmt.basketMenu, theConfig.size[0]-80-theConfig.xborder, 20 )
			elif meter == self.dispElmt.title:			# clic on the title
				theConfig.smallNote = not theConfig.smallNote
				karamba.writeConfigEntry(widget, "isReduced", str(theConfig.smallNote))
				if theConfig.smallNote:
					karamba.resizeWidget(widget, theConfig.size[0], 20)
				else:
					if self.skippedOnce:
						print "Reloading because the note was minimized for too long"
						self.reloadNotes(widget)
						karamba.redrawWidget(widget)
					karamba.resizeWidget(widget, theConfig.size[0], theConfig.size[1])
			elif meter == self.dispElmt.cfgButton:		# config button pressed
				karamba.popupMenu(widget, self.dispElmt.configMenu, theConfig.size[0]-100-theConfig.xborder, 20 )
			elif meter == self.myPostIts[self.currentPostIt].extendButton:	# extend long notes
				self.myPostIts[self.currentPostIt].extendNote(widget, 5000, " ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')
			else:		# highlight button pressed
				for kn in self.myPostIts[self.currentPostIt].notes:
					if meter == kn.hlButton:
						pos = karamba.getImagePos(widget, kn.hlButton)
						self.hlCurrentNote = kn
						karamba.popupMenu(widget, self.dispElmt.highlightMenu, pos[0], pos[1]+18)
		elif button==2:		#middle clic
			if meter == self.myPostIts[self.currentPostIt].extendButton:
				self.myPostIts[self.currentPostIt].extendNote(widget, 0, " ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')

	def commandOutput(self, widget, pid, output):

		if output =='\n':
			return

		# Change quick notes basket
		if pid == self.actions.select_quickBasket_pid:
			output = output[:-2].split('/');
			output = output[len(output)-1]
			theConfig.quickNotesBasket = output
			print "New Quick notes basket is", output
			self.cmpt_refresh = 1	# to refresh the display
			self.notesGatherer.needUpdate = True
			karamba.writeConfigEntry(widget, "quickNotesBasket", theConfig.quickNotesBasket)
		
		# ask for the text of the new note
		elif pid == self.actions.create_note_pid:
			
			try:
				print output
				output = unicode(output, 'utf-8')
			except:
				try:
					output = unicode(output)
				except:
					pass
			command = ['dcop', 'basket', 'BasketIface', 'createNoteHtml', output, theConfig.quickNotesBasket]
			self.cmpt_refresh = 1	# to refresh the display
			self.notesGatherer.needUpdate = True
			self.actions.importFile_pid = karamba.executeInteractive(widget, command)

		# Note created from dropped file ... or not
		elif pid == self.actions.importFile_pid:
			if output=='False':
				print 'Note creation failed'
				command = 'kdialog --caption DesktopBasket --sorry '+ _('Note creation failed')
				karamba.execute(command)
			self.cmpt_refresh = 1	# to refresh the display
			self.notesGatherer.needUpdate = True
			print 'Note creation succeeded', output

		# change background image
		elif pid == self.actions.select_img_pid:
			theConfig.bgImgFile = output
			karamba.deleteImage(widget, self.dispElmt.bgImg)
			self.dispElmt.bgImg = karamba.createBackgroundImage(widget, 0, 20, theConfig.bgImgFile)
			karamba.resizeImage(widget, self.dispElmt.bgImg, theConfig.size[0], theConfig.size[1]-20)
			karamba.writeConfigEntry(widget, "bgImg", theConfig.bgImgFile)
			karamba.redrawWidget(widget)

		# change title background image
		elif pid == self.actions.select_ttBar_pid:
			theConfig.ttBarImgFile = output
			karamba.deleteImage(widget, self.dispElmt.ttBarImg)
			self.dispElmt.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, theConfig.ttBarImgFile)
			karamba.resizeImage(widget, self.dispElmt.ttBarImg, theConfig.size[0], 20)
			karamba.writeConfigEntry(widget, "titleBar", theConfig.ttBarImgFile)
			karamba.redrawWidget(widget)

		# change title
		elif pid == self.actions.select_title_pid:
			theConfig.titleText = output[:-1]
			karamba.writeConfigEntry(widget, "title", theConfig.titleText)
			try:
				theConfig.titleText = unicode(theConfig.titleText, 'utf-8')
			except:
				try:
					theConfig.titleText = unicode(theConfig.titleText)
				except:
					pass
			karamba.changeText(widget, self.dispElmt.title, theConfig.titleText+" ("+ str(self.currentPostIt+1)+'/'+str(len(self.myPostIts))+')')
			theConfig.titleType= 'Text'
			karamba.writeConfigEntry(widget, "titleType", theConfig.titleType)
			karamba.setMenuConfigOption(widget, 'titleDate', False)
			karamba.redrawWidget(widget)

		# change text default color
		elif pid == self.actions.select_color_pid:
			theConfig.defaultColor = output[:-1]
			for c in Desktopbasket_config.colors.keys():
				if theConfig.defaultColor==_(c):
					theConfig.defaultColor=c
					break
			karamba.writeConfigEntry(widget, "defaultColor", theConfig.defaultColor)
			tmpSmall = theConfig.smallNote
			theConfig.smallNote = False
			self.notesGatherer.needUpdate=True
			karamba.changeTextColor(widget, self.dispElmt.title, int((Desktopbasket_config.colors[theConfig.defaultColor])[0:3]), int((Desktopbasket_config.colors[theConfig.defaultColor])[3:6]), int((Desktopbasket_config.colors[theConfig.defaultColor])[6:9]))
			karamba.redrawWidget(widget)
			theConfig.smallNote = tmpSmall

		# change size
		elif pid == self.actions.select_size_pid:
			s = output[:-1].lower()
			if s.find('x') == -1:
				return
			ss = s.split('x')
			theConfig.size[0]= int(ss[0])
			theConfig.size[1]= int(ss[1])
			karamba.resizeWidget(widget, theConfig.size[0], theConfig.size[1])
			karamba.resizeImage(widget, self.dispElmt.bgImg, theConfig.size[0], theConfig.size[1]-20)
			karamba.resizeImage(widget, self.dispElmt.ttBarImg, theConfig.size[0], 20)
			karamba.moveImage(widget, self.dispElmt.basketButton, theConfig.size[0]-16-theConfig.xborder, 2)
			karamba.moveImage(widget, self.dispElmt.cfgButton, theConfig.size[0]-36-theConfig.xborder, 2)
			karamba.resizeText(widget, self.dispElmt.title, theConfig.size[0]-100-2*theConfig.xborder, 20)
			karamba.writeConfigEntry(widget, "noteHeight", str(theConfig.size[1]))
			self.notesGatherer.needUpdate=True
			if theConfig.smallNote:
				karamba.resizeWidget(widget, theConfig.size[0], 20)
			self.notesGatherer.needUpdate=True
			self.reloadNotes(widget)
			karamba.redrawWidget(widget)

		# change selecting tag
		elif pid == self.actions.select_tag_pid:
			tmp = output[:-2].rsplit(' ')	# values are separated with spaces
			theConfig.desiredTag=[]
			for i in tmp:
				theConfig.desiredTag.append(i[1:-1])
			print "New preferred tags are", theConfig.desiredTag
			theConfig.saveDesiredTag(widget)
			self.cmpt_refresh = 1	# to refresh the display
			karamba.redrawWidget(widget)

		# change baskets names colour
		elif pid == self.actions.select_basketnamecolor_pid:
			color = output[:-1]
			if color==_('Don\'t show'):
				color='Don\'t show'
			else:
				for c in Desktopbasket_config.colors.keys():
					if color==_(c):
						color=c
						break
			theConfig.basketNameColor = color
			karamba.writeConfigEntry(widget, "basketNameColor", color)
			tmpSmall = theConfig.smallNote
			theConfig.smallNote = False
			self.notesGatherer.needUpdate=True
			self.reloadNotes(widget)
			karamba.redrawWidget(widget)
			theConfig.smallNote = tmpSmall

		# change refresh rate
		elif pid == self.actions.select_refresh_pid:
			theConfig.refreshRate = int(output[:-1].lower())
			karamba.writeConfigEntry(widget, "theConfig.refreshRate", str(theConfig.refreshRate))
	
	def onDrop(self, widget, dropText, x, y):
		if not isBasketRunning():
				command = ["kdialog", '--caption', 'DesktopBasket',"--error", _('Basket is not running')]
				karamba.executeInteractive(widget, command)
		else:
			#basketList = getoutput('dcop basket BasketIface listBaskets')
			if dropText.startswith('file:///') and os.path.isfile(dropText[7:]):
				print "Creating new note from dropped file"
				command = ['dcop', 'basket', 'BasketIface', 'createNoteFromFile', dropText, theConfig.quickNotesBasket]
			else:
				print "Creating new note from dropped text"
				command = ['dcop', 'basket', 'BasketIface', 'createNoteHtml', dropText, theConfig.quickNotesBasket]
			self.actions.importFile_pid = karamba.executeInteractive(widget, command)
	# allows the user to choose the basket in which quick notes are created
	def askForQuickNotesBasket(self, widget):
		global _
		command = ['kdialog', '--caption', 'DesktopBasket', '--radiolist', _('Select the basket in which quick notes will be created')]
		a = self.notesGatherer.basketsList
		for i in a:
			command.append(i.path)	# kdialog tag
			command.append(i.name)
			command.append('0')
		self.actions.select_quickBasket_pid = karamba.executeInteractive(widget, command)

	# allows the user to choose which tags make the notes display on the desktop
	def askForDesiredTag(self, widget):
		
		command = ['kdialog', '--caption', 'DesktopBasket', '--checklist', _('Select tags')]
		a = self.notesGatherer.choosingTags.keys()
		a.sort()
		for i in a:	# create list for kdialog
			command.append(self.notesGatherer.choosingTags[i])	# kdialog tag
			command.append(i)	# kdialog item
			if self.notesGatherer.choosingTags[i] in theConfig.desiredTag:
				command.append('on')	# select item in kdialog
			else:
				command.append('off')
		self.actions.select_tag_pid = karamba.executeInteractive(widget, command)

	# creates the desktop tag if it doesn't exist and adds it to the selecting tags
	def createDesktopTag(self, widget, askConfirm):
		global _
		tagsFile = open(theConfig.basketBase+'tags.xml', 'r')
		content = tagsFile.read()
		tagsFile.close()
		if re.search('<state id=\"desktop\"',content):
			text = _('The \"Desktop\" tag already exists')
			karamba.execute('kdialog --msgbox \''+ text + '\' --caption DesktopBasket')
		else:
			if askConfirm:
				ans = os.spawnlp( os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _('Do you really want to create \nthe \"Desktop\" tag in Basket ?'))
			else:
				ans=0
			if ans==0:
				dtag = '\n  <tag>\n  <name>'+str(_('Desktop'))+'</name>\n  <shortcut></shortcut>\n  <inherited>false</inherited>\n  <state id="desktop" >\n   <name>'+str(_('Desktop'))+'</name>\n   <emblem>desktop</emblem>\n   <text strikeOut="false" bold="false" underline="false" color="" italic="false" />\n   <font size="-1" name="" />\n   <backgroundColor></backgroundColor>\n   <textEquivalent string="" onAllTextLines="false" />\n  </state>\n </tag>\n</basketTags>'
				newcontent = content.replace('</basketTags>', dtag)
				tagsFile = open(theConfig.basketBase+'/tags.xml', 'w')
				tagsFile.write(newcontent)
				tagsFile.close()
				if getoutput('dcop | grep basket'):
					hidden = getoutput('dcop basket MainWindow hidden')
					ans = os.spawnlp(os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _('Basket needs to be restarted.\nDo you want to restart it now ?'), '--caption', 'DesktopBasket')
					if ans==0:
						os.spawnlp(os.P_WAIT, 'dcop', 'dcop', 'basket', 'MainApplication-Interface', 'quit')
						os.spawnlp(os.P_WAIT, 'basket', 'basket')
						if hidden=='true':
							karamba.execute('dcop basket MainWindow hide')
		if theConfig.desiredTag:
			if 'desktop' not in theConfig.desiredTag:	# if 'desktop' is already a selecting tag, don't append it to the list
				theConfig.desiredTag.append('desktop')
		else:
			theConfig.desiredTag = ['desktop']
		theConfig.saveDesiredTag(widget)


# Desktopbasket_config contains the configuration parameters for DesktopBasket
class Desktopbasket_config:

	#########################################################################
	#                       Theme's version                                 #
	#                    should be updated for each release                 #
	#########################################################################
	version = "1.5alpha3"

	authors = 'Romain Henriet (henriet.romain@libertysurf.fr)'
	contributors = 'Suslik (dd@accentsolution.com)'

	#_('white')
	# HTML colors
	colors = {'black':'000000000', 'white':'255255255', 'red':'255000000', 'blue':'000000255', 'yellow':'2552550', 'orange':'255165000', 'green':'000255000', 'purple':'160032240', 'maroon':'176048096', 'grey':'190190190', 'navy':'000000128', 'silver': '230230250'}

	def __init__(self, widget):

		print "Loading config"
		self.basketBase = self.searchForBasketBase()

		self.xborder=20
		self.yborder=12

		self.size=[0,0]
		
		self.smallNote = karamba.readConfigEntry(widget, "isReduced")=='True'

		self.showTags = karamba.readConfigEntry(widget, "showTags")!='False'
		
		self.desiredTag = karamba.readConfigEntry(widget, "desiredTag")
		if self.desiredTag:
			self.desiredTag=self.desiredTag.rsplit(';')

		# directory of the basket where to create quick notes
		self.quickNotesBasket = karamba.readConfigEntry(widget, "quickNotesBasket")

		self.titleType = karamba.readConfigEntry(widget, "titleType")
		if not self.titleType:
			self.titleType='Text'
			karamba.writeConfigEntry(widget, "titleType", self.titleType)
		if self.titleType=='Date':
			self.titleText = ""
		else:
			self.titleText = karamba.readConfigEntry(widget, "title")
			if not self.titleText:
				self.titleText = 'DesktopBasket'
				karamba.writeConfigEntry(widget, "title", self.titleText)
		
		self.size[0] = karamba.readConfigEntry(widget, "widgetWidth")
		if not self.size[0]:
			self.size[0]=350

		self.refreshRate = karamba.readConfigEntry(widget, "refreshRate")
		if not self.refreshRate:
			self.refreshRate=30

		self.basketNameColor = karamba.readConfigEntry(widget, "basketNameColor")
		if not self.basketNameColor:
			self.basketNameColor='Don\'t show'

		self.size[1] = karamba.readConfigEntry(widget, "noteHeight")
		if not self.size[1]:
			self.size[1]=200
			karamba.writeConfigEntry(widget, "noteHeight", str(self.size[1]))
			
		self.defaultColor = karamba.readConfigEntry(widget, "defaultColor")
		if not self.defaultColor:
			self.defaultColor = 'black'
			karamba.writeConfigEntry(widget, "defaultColor", self.defaultColor)
		else:
			try:
				self.titleText=unicode(self.titleText, 'utf8')
			except:
				try:
					self.titleText=unicode(self.titleText)
				except:
					pass

		self.resourcesPath = os.environ['HOME']+'/.superkaramba/desktopbasket'
		
		try:
			# i think open(blah).readline() closes the file auto, just like .read()
			# python is generally good at garbadge collection.
			self.installedVersion = open(self.resourcesPath + '/version').readline()
			print "DesktopBasket version is", self.installedVersion
		except:
			self.installedVersion = '0'

		themePath = karamba.getThemePath(widget)

		if self.installedVersion != Desktopbasket_config.version:	# using a new version

			if not os.path.exists(self.resourcesPath):
				os.makedirs(self.resourcesPath)
			if not themePath.endswith(".skz"):	# not running from a skz archive
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/images', self.resourcesPath)
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/editor', self.resourcesPath)
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/doc', self.resourcesPath)
			else:
				# extract image folder
				print '\"Images\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'images/*', '-d', self.resourcesPath)
				# extract editor folder
				print '\"Editor\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'editor/*', '-d', self.resourcesPath)
				# extract help folder
				print '\"Help\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'doc/*', '-d', self.resourcesPath)
			f=open(self.resourcesPath+'/version', 'w')
			f.write(Desktopbasket_config.version)
			f.close()
			# move old version's highlight file if there is one
			oldHlFilePath = os.environ['HOME']+'/.superkaramba/dektopbasket_highlights'
			if os.path.exists(oldHlFilePath):
				os.spawnlp(os.P_WAIT, 'mv', 'mv', oldHlFilePath, self.hlFilePath())

			self.installedVersion = Desktopbasket_config.version

			# initialize language and documentation
			lang = karamba.userLanguages(widget)
			langFile = None
			self.localeFile = ""
			docPath = self.resourcesPath+'/doc/en/index.docbook'
			for l in lang:
				fileContents = karamba.readThemeFile(widget, 'locales/'+l+'.mo')
				if fileContents.__len__() > 0:
					self.localeFile = l+'.mo'
					if os.path.exists(self.resourcesPath+'/doc/'+l+'/index.docbook'):
						docPath = self.resourcesPath+'/doc/'+l+'/index.docbook'
					break
			karamba.writeConfigEntry(widget, "locale", self.localeFile)
			os.chdir(self.resourcesPath+'/doc')
			os.spawnlp(os.P_WAIT, 'meinproc', 'meinproc', docPath)

		self.localeFile = karamba.readConfigEntry(widget, "locale")

		self.bgImgFile = karamba.readConfigEntry(widget, "bgImg")
		if not self.bgImgFile:
			self.bgImgFile = self.imagesPath()+'/background.png'
			karamba.writeConfigEntry(widget, "bgImg", self.bgImgFile)
			
		self.ttBarImgFile = karamba.readConfigEntry(widget, "titleBar")
		if not self.ttBarImgFile:
			self.ttBarImgFile = self.imagesPath()+'/TitleBar.png'
			karamba.writeConfigEntry(widget, "titleBar", self.ttBarImgFile)

		print "Config Loaded"

	def helpPath(self):
		return self.resourcesPath+'/doc/index.html'

	def editorPath(self):
		return self.resourcesPath+'/editor'

	def imagesPath(self):
		return self.resourcesPath+'/images'

	def hlFilePath(self):
		return self.resourcesPath+'/highlights'

	def searchForBasketBase(self):
		# a small rewrite. This takes care of more potential errors
		basketBase = ''
		a = getoutput('kde-config --localprefix')
		if a and not a.count('kde-config'): # if there was an error of any sort, that string will be in the responce.
			try:
				f = open(a+'share/config/basketrc').read() # this way you don't need to close it. does it by itself after read
				basketBase = re.search('(?<=dataFolder=).+',f).group().replace('$HOME',os.environ['HOME']) + '/'
				# this is a bit long, but if any part above fails, we don't need anyof this.
				# thus, I put it all under one try
			except:
				# I bet default basket install puts something there... so, something better than "return" with empty hands
				basketBase = a + 'share/apps/basket/'
				# we can also make it a '' and check for it later.
			print 'Basket baskets folder :', basketBase
		# i hope you do not mind my switching to return basketBase.
		# it makes it easier to reuse this function later somewhere
		return basketBase

	def saveDesiredTag(self, widget):
		# writes the list of tags that select notes to display into the config file
		selString = ''
		for t in self.desiredTag:
			selString = selString+t+';'		# separates tags with ';'
		selString = selString[:-1]
		karamba.writeConfigEntry(widget, "desiredTag", selString)

	
class DisplayElements:

	def __init__(self, widget):

		global theConfig

		self.widget= widget
		
		self.iconLoader = KGlobal.iconLoader()

		karamba.resizeWidget(widget, theConfig.size[0], theConfig.size[1])
		self.bgImg = karamba.createBackgroundImage(widget, 0, 20, theConfig.bgImgFile)
		karamba.resizeImage(widget, self.bgImg, theConfig.size[0],theConfig.size[1]-20)
		
		self.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, theConfig.ttBarImgFile)
		karamba.resizeImage(widget, self.ttBarImg, theConfig.size[0],20)
		
		self.moveLeftButton = karamba.createImage(widget, theConfig.xborder, 2, str(self.iconLoader.iconPath('1leftarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveLeftButton)
		
		self.moveRightButton = karamba.createImage(widget, 18+theConfig.xborder, 2, str(self.iconLoader.iconPath('1rightarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveRightButton)
		
		self.basketButton = karamba.createImage(widget, theConfig.size[0]-16-theConfig.xborder, 2, str(self.iconLoader.iconPath('basket', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.basketButton)
		
		self.cfgButton = karamba.createImage(widget, theConfig.size[0]-36-theConfig.xborder, 2, str(self.iconLoader.iconPath('configure', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.cfgButton)
		
		self.title = karamba.createText(widget, theConfig.xborder+40, -1, theConfig.size[0]-80-2*theConfig.xborder, 20, theConfig.titleText)
		karamba.attachClickArea(widget, self.title)
		karamba.changeTextColor(widget, self.title, 0, 0, 0)
		karamba.setTextAlign(widget, self.title, 'CENTER')
		karamba.changeTextSize(widget, self.title, 14)

		# Menu to use Basket
		self.basketOptions = {}
		self.basketMenu = karamba.createMenu(widget)
		self.basketOptions['createNote'] = karamba.addMenuItem(widget, self.basketMenu,_("Create Note"), 'filenew')
		karamba.addMenuSeparator(widget, self.basketMenu)
		self.basketOptions['runBasket'] = karamba.addMenuItem(widget, self.basketMenu,_("Run Basket"), 'basket')

		# Menu to configure DesktopBasket
		self.configOptions = {}
		self.configMenu = karamba.createMenu(widget)
		self.configOptions['refresh'] = karamba.addMenuItem(widget, self.configMenu,_("Refresh"), 'reload')
		self.configOptions['refreshRate'] = karamba.addMenuItem(widget, self.configMenu,_("Change refresh rate"), 'reload')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['selecttag'] = karamba.addMenuItem(widget, self.configMenu,_("Choose tags"), 'find')
		self.configOptions['createDesktopTag'] = karamba.addMenuItem(widget, self.configMenu,_("Create and use the \"Desktop\" tag"), 'desktop')
		self.configOptions['quickBasket'] = karamba.addMenuItem(widget, self.configMenu,_("Select quick notes basket"), 'knotes')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['titleDate'] = karamba.addMenuItem(widget, self.configMenu,_("Show/Hide hour in title bar"), 'history')
		self.configOptions['title'] = karamba.addMenuItem(widget, self.configMenu,_("Change title"), 'text')
		self.configOptions['color'] = karamba.addMenuItem(widget, self.configMenu,_("Change default text color"), 'colorize')
		self.configOptions['showTags'] = karamba.addMenuItem(widget, self.configMenu,_("Show/Hide tags"), 'idea')
		self.configOptions['basketNameColor'] = karamba.addMenuItem(widget, self.configMenu,_("Change basket name color"), 'colorize')
		self.configOptions['size'] = karamba.addMenuItem(widget, self.configMenu,_("Change note size"), 'crop')
		self.configOptions['titleBar'] = karamba.addMenuItem(widget, self.configMenu,_("Change title bar"), 'frame_image')
		self.configOptions['background'] = karamba.addMenuItem(widget, self.configMenu,_("Change background"), 'frame_image')
		self.configOptions['resetTheme'] = karamba.addMenuItem(widget, self.configMenu,_("Reset theme"), 'remove')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['help'] =  karamba.addMenuItem(widget, self.configMenu,_("Help"), 'help')
		self.configOptions['about'] = karamba.addMenuItem(widget, self.configMenu,_("About"), 'superkaramba')
		

		self.highlightOptions={}
		self.highlightMenu = karamba.createMenu(widget)
		a = os.listdir(theConfig.imagesPath()+'/highlight')
		self.highlightOptions["None"] = karamba.addMenuItem(widget, self.highlightMenu, '', theConfig.imagesPath()+'/highlight/None')
		for i in a:
			if i.endswith('.png'):
				self.highlightOptions[i] = karamba.addMenuItem(widget, self.highlightMenu, '', theConfig.imagesPath()+'/highlight/'+i)

		self.highlightIcon = str(self.iconLoader.iconPath('bookmark', KIcon.Small, False))

		self.dropArea = karamba.acceptDrops(widget)

# Desktopbasket_actions contains actions's id to identify clicked meters
class Desktopbasket_actions:

	def __init__(self):
		self.select_img_pid = None
		self.select_title_pid = None
		self.select_size_pid = None
		self.select_color_pid = None
		self.select_ttBar_pid = None
		self.select_tag_pid = None
		self.select_refresh_pid = None
		self.select_basketnamecolor_pid = None
		self.create_note_pid = None	# after the basket has been chosen
		self.importFile_pid = None
		self.select_quickBasket_pid = None

# HTML Parser to read notes content
class MyParser(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.reset()
	
	def handle_starttag(self, tag, attrs):
		if tag == 'body':
			self.start = self.getpos()

	def handle_endtag(self, tag):
		if tag == 'body':
			self.end=self.getpos()
			self.end = (self.end[0], self.end[1]+7)

	def getPositions(self):
		return self.start, self.end

	def reset(self):
		HTMLParser.reset(self)
		self.content = ''
		self.start = None
		self.end = None

# Searches for baskets, parses their contents and 
# reads notes
class NotesGatherer:

	def __init__(self):
		self.lastTagsChange = None
		self.choosingTags = {}
		self.basketsList=[]
		self.needUpdate = True
		self.desiredTag = theConfig.desiredTag
		self.basketsMonitor = gamin.WatchMonitor()
		self.notesMonitor = gamin.WatchMonitor()
		self.tagsMonitor = gamin.WatchMonitor()
		self.basketsMonitor.watch_file(theConfig.basketBase+"baskets/baskets.xml", self.empty_callback)
		self.tagsMonitor.watch_file(theConfig.basketBase+"tags.xml", self.tags_callback)
		self.reloadTags()
		
	def main(self):
		#self.needUpdate = False
		self.notesList=[]
		self.myHTMLParser = MyParser()
		#self.getBaskets()

		if self.tagsMonitor.event_pending():
			self.tagsMonitor.handle_one_event()
		# baskets have changed
		if self.basketsMonitor.event_pending():
			self.getBaskets()
			print "Change in baskets===="
			self.needUpdate=True
			self.basketsMonitor.handle_events()

		# desired tag has changed
		if self.desiredTag!=theConfig.desiredTag:
			print "Change in tags===="
			self.needUpdate=True
			self.desiredTag=theConfig.desiredTag

		if not self.needUpdate:	
			#for b in self.basketsList:
				#d = os.stat(b.path).st_mtime
				#if d!=b.timestamp:
					#self.needUpdate=True
					#b.timestamp=d
			#if not self.needUpdate:
			if self.notesMonitor.event_pending():
				self.notesMonitor.handle_events()
				print "Change in notes===="
				self.needUpdate=True
			else:
				return self.notesList	#no need to retrieve notes since nothing has changed
		self.notesMonitor.handle_events()
		self.getNotes()
		#print self.notesList
		self.readNotes()
		return self.notesList

	def empty_callback(self, path, event):
		#print 'callback:', path, event
		pass

	# gets baskets list
	def getBaskets(self):
		#stat = os.stat(theConfig.basketBase+"baskets/baskets.xml")
		
		#if self.lastBasketListChange==stat.st_mtime:
			#print 'Basket list didn\'t change'
			#return
		self.needUpdate = True
		for b in self.basketsList:
			self.notesMonitor.stop_watch(b.path+'.basket')		#stop watching notes changes
		self.basketsList=[]
		#self.lastBasketListChange = stat.st_mtime
		#print 'Looking for baskets'
		doc = parse (theConfig.basketBase+"baskets/baskets.xml")
		baskets = doc.getElementsByTagName("basket")
		
		for b in baskets :
			folder = b.attributes.get("folderName", None)
			prot = b.getElementsByTagName('protection')[0]
			if prot.attributes.get('type').value != '0': # basket is protected
				continue # skip it
			name = b.getElementsByTagName('name')[0]
			name=name.childNodes[0].nodeValue
			#print name + '==================================='
			btmp = Basket(theConfig.basketBase+'baskets/'+folder.value, name)
			self.basketsList.append( btmp)
			self.notesMonitor.watch_file(theConfig.basketBase+'baskets/'+folder.value+'.basket', self.empty_callback)
			#print theConfig.basketBase+'baskets/'+folder.value+'.basket'
			#btmp.timestamp = 0;
		print len(self.basketsList), 'baskets found'

	# extracts notes from the baskets
	def getNotes(self):
		print 'Looking for notes'
		for B in self.basketsList:		# get notes in each basket
			b=B.path
			#print 'in',b
			try:
				doc = parse ( b + u'.basket')
			except:
				print "Error while parsing basket", b
			notes = doc.getElementsByTagName("note")

			#if len(self.desiredTag)==1:
				#hideDesiredTag=True
			#else:
				#hideDesiredTag=False

			qBasket = b.split('/')
			qBasket = qBasket[len(qBasket)-2]==theConfig.quickNotesBasket	# true if quick notes basket

			for n in notes:

				## choose which notes to keep
				typeN = n.attributes.get("type", None)	# read note type (html, image, ...)
				if typeN == None:
					continue	# no type ? skip the note
				typeN = typeN.value
				if typeN != u'html' and typeN != u'image' and typeN!= u'launcher' and typeN!=u'file' and typeN != u'link':	# skip unmanaged notes
					continue	# this type is not managed by desktopbasket : skip the note
				tag = n.getElementsByTagName("tags")	# read the note's tags
				if len(tag)==0:
					if qBasket:	# even if notes in "Quick notes basket" don't have tags, keep them
						tags = []
					else:
						continue	# no tag ? skip the note
				else:
					tags = tag[0].childNodes[0].nodeValue
					tags = tags.rsplit(';')		# split list of tags
				skipNote = True

				if not qBasket:
					for seltag in self.desiredTag:
						if  seltag in tags:	# skip the note if there isn't one the desired tags
							skipNote = False
							break
				else:
					skipNote = False	# keep the note if in "Quick notes basket"

				if skipNote:
					continue

				## now, we only have notes that will be displayed

				ct = n.getElementsByTagName("content")[0]
				c =ct.childNodes[0]
				#switch according to note's type
				if typeN == u'html': # text
					newNote = Note(b+c.nodeValue, 'text', B)
				elif typeN == u'link':	# link
					linkText = ct.attributes.get("title")
					linkText = linkText.childNodes[0].nodeValue
					command = 'kfmclient exec \''+c.nodeValue+'\''
					newNote = Note('', 'link', B, command)
					newNote.text = '<p><u>'+linkText+'</u></p>'
				elif typeN == u'launcher':	# launcher
					serv = KService(b+c.nodeValue)
					command = 'kfmclient exec \''+b+c.nodeValue+'\''
					newNote = Note(b+c.nodeValue, 'launcher', B, command)
					newNote.text = '<p><img src="'+findIconPath(str(serv.icon()), KIcon.NoGroup)+'" height="24" align ="left" />&nbsp;''</p>'
				elif typeN == u'file' : # a file
					fi = KFileItem (0, 0, KURL(b+c.nodeValue), False)
					command = 'kfmclient exec \''+b+c.nodeValue+'\''
					newNote = Note(b+c.nodeValue, 'file', B, command)
					newNote.text = '<p><img src="'+findIconPath(str(fi.iconName()), KIcon.NoGroup)+'" height="20" align ="left" />&nbsp;'+c.nodeValue+'</p>'
				elif typeN == u'image': # image
					newNote = Note(b+c.nodeValue, 'img', B)
					newNote.text = '<img src="'+b+c.nodeValue+'" height="54" />'
				for t in tags:	# save the note's tags
					if self.tags.has_key(t):
						newNote.tags.append(self.tags[t])
				self.notesList.append(newNote)
		print len(self.notesList),"notes found"

	# reads notes contents
	def readNotes(self):
		try:
			hlFile = open (theConfig.hlFilePath(), 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except IOError, e:
			highlights = {}
		highlightsNew = {}
		#print 'Reading notes content'
		text = ''
		for n in self.notesList:
			#print "Reading", n
			if n.path==None or not os.path.isfile(n.path) :
				continue

			if n.type == 'text':	# text note	
				doc = open (n.path)
				data = doc.read()
				doc.close()
				doc = open (n.path)
				self.myHTMLParser.reset()
				self.myHTMLParser.feed(data)
				self.myHTMLParser.close()
				c = self.myHTMLParser.getPositions()
				aa = self.readPartOfFile(doc, c[0], c[1])
				if not aa:
					aa=' '
				n.text = aa
				#print n.text, '=================='
				try:
					n.text = unicode(aa, 'utf-8')
				except:
					print "Error while decoding note(utf8)"
					try:
						n.text = unicode(aa)
					except:
						print "Error while decoding note"
				doc.close()

			# highlight the note if needed
			if highlights.has_key(n.path):
				tmp = highlights[n.path]
				highlightsNew[n.path]=tmp
				n.isHighlighted=tmp

		try:
			hlFile = open (theConfig.hlFilePath(), 'wb')
			marshal.dump(highlightsNew, hlFile)
			hlFile.close()
		except:
			pass
		try:
			hlFile = open (theConfig.hlFilePath(), 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except IOError, e:
			highlights = {}
		print 'ok, all notes read'

	# reads file from start position to end position
	def readPartOfFile(self, doc, start, end):
		#print start, end
		if (not start) or (not end):
			return
		for i in range(start[0]-1):
			doc.readline()
		text = ''
		if start[0] < end[0]:
			for i in range(start[0], end[0]):
				text = text + doc.readline()
		#print text
		lastLine = doc.readline()
		#print lastLine, start, end, '________________________'
		lastLine = lastLine[:end[1]]
		text = text + lastLine
		return text[start[1]:]

	def tags_callback(self, path, event):
		self.needUpdate = True
		self.reloadTags()

	# reloads list of tags
	def reloadTags(self):
		if not os.path.isfile(theConfig.basketBase+ 'tags.xml'):
			print "Tags file not found !"
			self.tags={}
			return
		print 'Loading tags'
		self.tags = {}
		doc = parse (theConfig.basketBase+ 'tags.xml')
		#print 'Ok, tags file parsed'
		tags = doc.getElementsByTagName('tag')
		#print "Found", len(tags), "tags"
		for t in tags:
			states = t.getElementsByTagName("state")
			#print "Found", len(states), "states"
			for st in states:
				tagId = st.attributes.get('id').value
				name=[]
				if len(states)==1:
					name = t.getElementsByTagName("name")[0].childNodes
				else:
					try:
						name = st.getElementsByTagName("name")[0].childNodes
					except:
						pass
				if name==[]:
					name = tagId
				else:
					name = name[0].nodeValue
				self.choosingTags[name]=tagId
				#print name, '=====>', tagId
				icon = st.getElementsByTagName("emblem")
				if icon==[]:
					continue				
				icon = icon[0].childNodes
				if len(icon) != 0:
					iconName = icon[0].nodeValue
					self.tags[tagId] = findIconPath(iconName, KIcon.Small)

# initialize locale
def init_language(widget):

	global _, theConfig
	
	#helpPath = theConfig.helpPath
	if theConfig.localeFile!="":
		fileContents = karamba.readThemeFile(widget, 'locales/'+theConfig.localeFile)
		gt = GNUTranslations(StringIO(fileContents))
		_ = gt.ugettext

# return true if basket is running
def isBasketRunning():
	return getoutput('dcop | grep basket')

# launch basket
def runBasket():
	print 'Running basket'
	if getoutput('dcop | grep basket'):
		karamba.execute('dcop basket MainWindow restore')
	else:
		karamba.execute('basket')

# returns the path to an icon
def findIconPath(iconName, size):
	iconLoader = KGlobal.iconLoader()
	if iconName.startswith('/'):
		return iconName
	else:
		return str(iconLoader.iconPath(iconName, size, False))
