#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    DesktopBasket is free software; you can redistribute it and/or modify #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    DesktopBasket is distributed in the hope that it will be useful,      #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# This file contains data structures, such as Basket, Note, karambaNote, ...

import karamba, marshal, re

try:
	from kdecore import KIcon
	from kdeui import KPassivePopup
	from qt import QPixmap
except ImportError, e:
	comm = "kdialog --caption DesktopBasket --error \"Failed to load a module, check you installation : \n" + str(e) + "\""
	karamba.execute(comm)
	sys.exit()

# A basket in Basket Note Pads
class Basket:

	def __init__(self, file, name):
		self.path = file
		self.name = name
		#self.timestamp = None

# A note is a data contained in a basket
class Note:

	def __init__(self, path, type, basket, action=None):
		self.text = None
		self.action = action	# action to perform when the note is clicked
		self.basket = basket	# container
		self.tags = []	# note's tags in Basket
		self.path = path	# path in the file system
		self.isHighlighted = 'None'	# highlight color
		self.type = type	# a note can be an image, a text, ...

	def getFormattedText(self, color):
		#print self.tags
		return '<font color=\"'+color+'\"> '+ self.text + '</font>'

# A karambaNote contains properties to display a note
class karambaNote:

	def __init__(self, widget, note, ypos, config, dispElmt):
		self.config=config
		self.clickarea = None
		self.note = note	# Note to display
		self.rt = karamba.createRichText(widget, note.getFormattedText(self.config.defaultColor))
		dim_text = karamba.getRichTextSize(widget, self.rt)
		displayedTags = []
		self.icons = []
		if config.showTags:
			# optimize space to display text and tags
			nb_tags = len(note.tags)
			S = dim_text[0]*dim_text[1] + nb_tags * 400		# superficy of the note
			k = S/((self.config.size[0]-2*self.config.xborder-18)*20)	# number of lines
			if k==0:
				k=1
			kk = int(nb_tags/k)+1		# number of tags in one line
		
			karamba.setRichTextWidth(widget, self.rt, self.config.size[0]-kk*18-2*self.config.xborder-18)
			karamba.moveRichText(widget, self.rt, self.config.xborder+18, ypos)
			cmpt_v = 0
			xpos = self.config.size[0]- 16 - self.config.xborder

			for i in note.tags:
				image = karamba.createImage(widget, xpos, ypos + 18*cmpt_v +3 , i)
				karamba.resizeImage(widget, image, 16, 16)	# for custom icons
				if cmpt_v==k-1:
					cmpt_v = 0
					xpos = xpos - 18
				else:
					cmpt_v = cmpt_v+1
				#print i
				self.icons.append(image)
		else:	# only display text, disposition is easier
			karamba.setRichTextWidth(widget, self.rt, self.config.size[0]-2*self.config.xborder-18)
			karamba.moveRichText(widget, self.rt, self.config.xborder+18, ypos)

		self.highlight=None
		if note.isHighlighted:
			h = karamba.getRichTextSize(widget, self.rt)[1]
			self.highlight = karamba.createImage(widget, self.config.xborder, ypos+1, self.config.imagesPath()+'/highlight/'+note.isHighlighted)
			karamba.resizeImage(widget, self.highlight, self.config.size[0]-2*self.config.xborder, h+1)
		self.hlButton = karamba.createImage(widget, self.config.xborder, ypos+3, dispElmt.highlightIcon)
		karamba.attachClickArea(widget, self.hlButton)

	def createClickAction(self, widget):
		if not self.clickarea:	# create
			pos = karamba.getRichTextPos(widget, self.rt)
			size = karamba.getRichTextSize(widget, self.rt)
			self.clickarea = karamba.createClickArea(widget, pos[0], pos[1], size[0], size[1], self.note.action)
		else:		# delete
			karamba.removeClickArea(widget, self.clickarea)
			self.clickarea = None

	def changeHighlight(self, widget, color):
		if self.note.isHighlighted==color:	#no change
			return
		if self.highlight:
			karamba.deleteImage(widget, self.highlight)
			self.highlight=None
		try:
			hlFile = open (self.config.hlFilePath(), 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except IOError, e:
			highlights={}
		if color=='None':
			self.note.isHighlighted='None'
			if highlights.has_key(self.note.path):
				del highlights[self.note.path]
		else:
			self.note.isHighlighted=color
			highlights[self.note.path]=color
			xy = karamba.getRichTextPos(widget, self.rt)
			self.highlight = karamba.createImage(widget, self.config.xborder, xy[1]+1, self.config.imagesPath()+'/highlight/'+color)
			xy = karamba.getRichTextSize(widget, self.rt)
			karamba.resizeImage(widget, self.highlight, self.config.size[0]-2*self.config.xborder, xy[1]+1)
			karamba.attachClickArea(widget, self.hlButton)
		try:
			hlFile = open (self.config.hlFilePath(), 'wb')
			marshal.dump(highlights, hlFile)
			hlFile.close()
		except IOError, e:
			pass

	def move(self, widget, ypos):
		xy = karamba.getRichTextPos(widget, self.rt)
		diff_y = ypos-xy[1]
		karamba.moveRichText(widget, self.rt, xy[0], diff_y+xy[1])
		for i in self.icons:
			xy = karamba.getImagePos(widget, i)
			karamba.moveImage(widget, i, xy[0], diff_y+xy[1])
		if self.highlight:
			xy = karamba.getImagePos(widget, self.highlight)
			karamba.moveImage(widget, self.highlight, xy[0], diff_y+xy[1])
		if self.hlButton:
			xy = karamba.getImagePos(widget, self.hlButton)
			karamba.moveImage(widget, self.hlButton, xy[0], diff_y+xy[1])
		if self.clickarea:
			self.createClickAction(widget)	# delete
			self.createClickAction(widget)	# create
			
	def delete(self, widget):
		for i in self.icons:
			karamba.deleteImage(widget, i)
		if self.highlight:
			karamba.deleteImage(widget, self.highlight)
			self.highlight=None
		karamba.deleteRichText(widget, self.rt)
		if self.hlButton:
			karamba.deleteImage(widget, self.hlButton)
		if self.clickarea:
			self.createClickAction(widget)	#delete
			
	def show(self, widget, s):
		if s:
			karamba.showRichText(widget, self.rt)
			for i in self.icons:
				karamba.showImage(widget, i)
			if self.highlight:
				karamba.showImage(widget, self.highlight)
			if self.hlButton:
				karamba.showImage(widget, self.hlButton)
			if self.note and self.note.action and not self.clickarea:
				self.createClickAction(widget)	#create
		else:
			karamba.hideRichText(widget, self.rt)
			for i in self.icons:
				karamba.hideImage(widget, i)
			if self.highlight:
				karamba.hideImage(widget, self.highlight)
			if self.hlButton:
				karamba.hideImage(widget, self.hlButton)
			if self.clickarea:
				self.createClickAction(widget)

# Baskets'names are considered to be karambaNotes
class karambaBasketName(karambaNote):

	def __init__(self, widget, note, ypos, config):
		self.note = None
		self.icons=[]
		self.highlight=None
		self.hlButton=None
		self.clickarea=None
		self.config = config
		self.rt = karamba.createRichText(widget, '<font color=\"'+self.config.basketNameColor+'\"><h3>' + note.basket.name + '</h3></font>')
		karamba.moveRichText(widget, self.rt, self.config.xborder, ypos)
		karamba.setRichTextWidth(widget, self.rt, self.config.size[0]-2*self.config.xborder)

# A PostIt is a page in DesktopBasket
# It contains several karambaNotes that are displayed in the same time
class PostIt:

	def __init__(self, config):
		self.notes = []	# notes appearing on the post-it
		self.extendButton=None	# button to display big notes
		self.config = config

	def append(self, kn):
		self.notes.append(kn)

	def show(self, widget, s):
		for n in self.notes:
			n.show(widget, s)
		if self.extendButton != None :
			if s:
				karamba.showImage(widget, self.extendButton)
			else:
				karamba.hideImage(widget, self.extendButton)

	def delete(self, widget):
		for n in self.notes:
			n.delete(widget)
		if self.extendButton != None :
			karamba.deleteImage(widget, self.extendButton)

	def setExtendable(self, widget, iconLoader):

		if len(self.notes)==1:
			cutNote = self.notes[0]
		elif len(self.notes)==2:
			cutNote = self.notes[1]
		else:
			return
		t = cutNote.note.getFormattedText(self.config.defaultColor)
		s = karamba.getRichTextSize(widget, cutNote.rt)
		limit = -karamba.getRichTextPos(widget, cutNote.rt)[1]+self.config.size[1]-self.config.yborder-15

		# remove the end of the note so that it doesn't go over the edge of the post-it
		cut_pos = int((self.config.size[1]+50)*len(t)/s[1])
		while True:
			text = t[:cut_pos]	# the cut is ugly because we lose the end of some html tags, but it works
			end_of_tag = re.compile('^[^<]*>').match(t[cut_pos:])
			if end_of_tag:
				text = text + end_of_tag.group()
			text = text + ' ...</li></ul></span></p></body></html>'
			#print text
			karamba.changeRichText(widget, cutNote.rt, text)
			if karamba.getRichTextSize(widget, cutNote.rt)[1] < limit:
				break
			cut_pos = cut_pos-20
		
		self.extendButton = karamba.createImage(widget, self.config.xborder, self.config.size[1]-60, str(iconLoader.iconPath('viewmag', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.extendButton)

	def extendNote(self, widget, timeout, endtext ):
		if len(self.notes) != 1:
			return
		text = self.notes[0].note.getFormattedText(self.config.defaultColor)
		pop = KPassivePopup.message(KPassivePopup.Balloon, "DesktopBasket"+endtext, text, QPixmap(), None, '', timeout)








	