#!/usr/bin/env python
# -*- coding: utf-8 -*-
#######################################################
#    Copyright (C) 2007 by Romain Henriet                                                                    #
#    henriet.romain@libertysurf.fr                                                                                  #
#                                                                                                                                    #
#    DesktopBasket is free software; you can redistribute it and/or modify                     #
#    it under the terms of the GNU General Public License as published by                    #
#    the Free Software Foundation; either version 2 of the License, or                            #
#    (at your option) any later version.                                                                            #
#                                                                                                                                    #
#    DesktopBasket is distributed in the hope that it will be useful,                               #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
#    GNU General Public License for more details.                                                         #
#                                                                                                                                   #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, write to the                                                           #
#    Free Software Foundation, Inc.,                                                                              #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                                 #
######################################################

import sys, os, gettext
from kdeui import *
from kdecore import *
from qt import QPixmap, QTextEdit, SIGNAL

# Initialize localization
pathname = os.path.dirname(sys.argv[0])
pathname = os.path.abspath(pathname)
gettext.install("messages", pathname + "/locale", unicode=1)

class Edit(KMainWindow):
	
	def __init__ (self):
		KMainWindow.__init__(self)
		self.view=KTextEdit(self, "Main View")
		self.view.setAlignment(QTextEdit.AlignLeft)
		self.setCaption(_('Notes Editor'))
		icon = QPixmap(pathname+'/icon.png')
		self.setIcon(icon)
		self.setCentralWidget(self.view)
		self.view.setTextFormat(QTextEdit.RichText)
		self.initToolBar()
		self.initStatusBar()
		self.connect(self.view, SIGNAL("selectionChanged()"), self.selectionChanged)
		self.resize(400, 250)
		self.show()
		
	# create toolbar with format buttons
	def initToolBar(self):
		
		self.toolbar1 = KToolBar(self, "EditBar")
		self.toolbar2 = KToolBar(self, "EditBar2")
		self.toolbar3 = KToolBar(self, "QuitBar")
		
		#undo/redo
		self.undoAction = KStdAction.undo(self.view.undo, self.actionCollection())
		self.undoAction.plug(self.toolbar1)
		self.undoAction.setEnabled(False)
		self.redoAction = KStdAction.redo(self.view.redo, self.actionCollection())
		self.redoAction.plug(self.toolbar1)
		self.redoAction.setEnabled(False)
		
		self.connect(self.view, SIGNAL('undoAvailable (bool)'), self.undoAv)
		self.connect(self.view, SIGNAL('redoAvailable (bool)'), self.redoAv)
		
		#alignment
		self.alignLeftAction = KToggleAction(_("Align left"), "text_left", KShortcut(), self)
		self.alignCenterAction = KToggleAction(_("Align center"), "text_center", KShortcut(), self)
		self.alignRightAction = KToggleAction(_("Align right"), "text_right", KShortcut(), self)
		self.alignBlockAction = KToggleAction(_("Align block"), "text_block", KShortcut(), self)
		self.alignLeftAction.plug(self.toolbar1)
		self.connect(self.alignLeftAction, SIGNAL("toggled(bool)"), self.textAlignLeft)
		self.alignCenterAction.plug(self.toolbar1)
		self.connect(self.alignCenterAction, SIGNAL("toggled(bool)"), self.textAlignCenter)
		self.alignRightAction.plug(self.toolbar1)
		self.connect(self.alignRightAction, SIGNAL("toggled(bool)"), self.textAlignRight)
		self.alignBlockAction.plug(self.toolbar1)
		self.connect(self.alignBlockAction, SIGNAL("toggled(bool)"), self.textAlignBlock)
		
		self.alignLeftAction.setExclusiveGroup('align')
		self.alignCenterAction.setExclusiveGroup('align')
		self.alignRightAction.setExclusiveGroup('align')
		self.alignBlockAction.setExclusiveGroup('align')
		
		#style
		self.textBoldAction = KToggleAction(_("Bold"), "text_bold", KShortcut(), self)
		self.textItalicAction = KToggleAction(_("Italic"), "text_italic", KShortcut(), self)
		self.textUnderlineAction = KToggleAction(_("Underline"), "text_under", KShortcut(), self)
		self.textSuper= KToggleAction(_("Superscript"), "text_super", KShortcut(), self)
		self.textSub= KToggleAction(_("Subscript"), "text_sub", KShortcut(), self)
		
		self.textBoldAction.plug(self.toolbar1)
		self.connect(self.textBoldAction, SIGNAL("toggled(bool)"), self.view.setBold)
		self.textItalicAction.plug(self.toolbar1)
		self.connect(self.textItalicAction, SIGNAL("toggled(bool)"), self.view.setItalic)
		self.textUnderlineAction.plug(self.toolbar1)
		self.connect(self.textUnderlineAction, SIGNAL("toggled(bool)"), self.view.setUnderline)
##		self.textSuper.plug(self.toolbar1)
##		self.connect(self.textSuper, SIGNAL("toggled(bool)"), self.vtextAlign)
##		self.textSub.plug(self.toolbar1)
##		self.connect(self.textSub, SIGNAL("toggled(bool)"), self.vtextAlign)
##		self.textSuper.setExclusiveGroup("valign")
##		self.textSub.setExclusiveGroup("valign")
	
		# Font
		self.fontNameAction = KFontAction(_("Text Font"), "text", KShortcut(), self)
##		self.fontNameAction.setFont(self.view.family())
		self.fontSizeAction = KFontSizeAction(_("Text Size"), KShortcut(), self)
##		self.fontSizeAction.setFontSize(self.view.pointSize())
		
		
		self.fontNameAction.plug(self.toolbar2)
		self.connect(self.fontNameAction, SIGNAL('activated()'), self.fontChanged)
		self.fontSizeAction.plug(self.toolbar2)
		self.connect(self.fontSizeAction, SIGNAL('activated()'), self.fontChanged)
		
		self.fontColorCombo = KColorCombo(self.toolbar2)
##		self.fontColorCombo.setColor(self.view.color())
		self.fontColorCombo.setFixedWidth(50)
		self.connect(self.fontColorCombo, SIGNAL('activated(const QColor &)'), self.fontChanged)
		
		#Quit Buttons
		self.okAction = KAction(_('Ok'), "ok", KShortcut(), self)
		self.cancelAction = KAction(_('Cancel'), "cancel", KShortcut(), self)
		self.okAction.plug(self.toolbar3)
		self.cancelAction.plug(self.toolbar3)
		self.connect(self.okAction, SIGNAL('activated()'), self.endEdition)
		self.connect(self.cancelAction, SIGNAL('activated()'), self.cancelEdition)
		
		self.selectionChanged()
	
	# Enable undo action when possible
	def undoAv(self):
		if self.view.isUndoAvailable():
			self.undoAction.setEnabled(True)
		else:
			self.undoAction.setEnabled(False)
	
	# Enable redo action when possible
	def redoAv(self):
		if self.view.isRedoAvailable():
			self.redoAction.setEnabled(True)
		else:
			self.redoAction.setEnabled(False)
		
	def cancelEdition(self):
		self.close()
	
	def endEdition(self):
		print unicode(self.view.text()).encode('utf-8')
		self.close()
	
	def textAlignLeft(self):
		if self.alignLeftAction.isChecked():
			self.view.setAlignment(QTextEdit.AlignLeft)
		
	def textAlignCenter(self):
		if self.alignCenterAction.isChecked():
			self.view.setAlignment(QTextEdit.AlignCenter)

	def textAlignRight(self):
		if self.alignRightAction.isChecked():
			self.view.setAlignment(QTextEdit.AlignRight)
			
	def textAlignBlock(self):
		if self.alignBlockAction.isChecked():
			self.view.setAlignment(QTextEdit.AlignJustify)
		
	def initStatusBar(self):
		self.statusBar().insertItem(_("Edit your note"),0)
		
##	def vtextAlign(self):
##		if self.textSub.isChecked():
##			self.view.setVerticalAlignment(QTextEdit.AlignSubScript)
##		elif self.textSuper.isChecked():
##			self.view.setVerticalAlignment(QTextEdit.AlignSuperScript)
##		else:
##			self.view.setVerticalAlignment(QTextEdit.AlignNormal)
			
	def fontChanged(self):
		self.view.setFamily(self.fontNameAction.font())
		self.view.setPointSize(self.fontSizeAction.fontSize())
		self.view.setColor(self.fontColorCombo.color())
		
	# Updates buttons when selection changes
	def selectionChanged(self):
		self.textBoldAction.setChecked(self.view.bold())
		self.textItalicAction.setChecked(self.view.italic())
		self.textUnderlineAction.setChecked(self.view.underline())
		self.fontNameAction.setFont(self.view.family())
		self.fontSizeAction.setFontSize(self.view.pointSize())
		self.fontColorCombo.setColor(self.view.color())
		al = self.view.alignment()
		if al==QTextEdit.AlignLeft:
			self.alignLeftAction.setChecked(True)
		elif al==QTextEdit.AlignCenter:
			self.alignCenterAction.setChecked(True)
			self.alignLeftAction.setEnabled(False)
		elif al==QTextEdit.AlignRight:
			self.alignRightAction.setChecked(True)
		elif al==QTextEdit.AlignJustify:
			self.alignBlockAction.setChecked(True)

app=KApplication(sys.argv,"NotesEditor")
toplevel=Edit()
app.setMainWidget(toplevel)
toplevel.show()
app.exec_loop()
